# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .user import User
from .._models import BaseModel

__all__ = ["UserCreateResponse"]


class UserCreateResponse(BaseModel):
    data: User
    """User object"""
